@@ -43,48 +43,120 @@
 
 using namespace KMime;
 
-//@cond PRIVATE
-int DateFormatter::mDaylight = -1;
-//@endcond
-DateFormatter::DateFormatter(FormatType ftype)
-    : mFormat(ftype), mTodayOneSecondBeforeMidnight(0)
+namespace KMime {
+
+class DateFormatterPrivate {
+public:
+    DateFormatterPrivate() :
+        mTodayOneSecondBeforeMidnight(0)
+    {}
+
+    /**
+      Returns a QString containing the specified time_t @p t formatted
+      using the #Fancy #FormatType.
+
+      @param t is the time_t to use for formatting.
+    */
+    QString fancy(time_t t);
+
+    /**
+      Returns a QString containing the specified time_t @p t formatted
+      using the #Localized #FormatType.
+
+      @param t is the time_t to use for formatting.
+      @param shortFormat if true, create the short version of the date string.
+      @param lang is a QString containing the language to use.
+    */
+    static QString localized(time_t t, bool shortFormat = true, const QString &lang = QString());
+
+    /**
+      Returns a QString containing the specified time_t @p t formatted
+      with the ctime() function.
+
+      @param t is the time_t to use for formatting.
+    */
+    static QString cTime(time_t t);
+
+    /**
+      Returns a QString containing the specified time_t @p t in the
+      "%Y-%m-%d %H:%M:%S" #Iso #FormatType.
+
+      @param t is the time_t to use for formatting.
+    */
+    static QString isoDate(time_t t);
+
+    /**
+      Returns a QString containing the specified time_t @p t in the
+      #Rfc #FormatType.
+
+      @param t is the time_t to use for formatting.
+    */
+    static QString rfc2822(time_t t);
+
+    /**
+      Returns a QString containing the specified time_t @p t formatted
+      with a previously specified custom format.
+
+      @param t time used for formatting
+    */
+    QString custom(time_t t) const;
+
+    /**
+      Returns a QString that identifies the timezone (eg."-0500")
+      of the specified time_t @p t.
+
+      @param t time to compute timezone from.
+    */
+    static QByteArray zone(time_t t);
+
+    DateFormatter::FormatType mFormat;
+    time_t mTodayOneSecondBeforeMidnight;
+    QString mCustomFormat;
+};
+
+}
+
+DateFormatter::DateFormatter(FormatType ftype) :
+    d(new DateFormatterPrivate)
 {
+    d->mFormat = ftype;
 }
 
 DateFormatter::~DateFormatter()
 {
+    delete d;
 }
 
 DateFormatter::FormatType DateFormatter::format() const
 {
-    return mFormat;
+    return d->mFormat;
 }
 
 void DateFormatter::setFormat(FormatType ftype)
 {
-    mFormat = ftype;
+    d->mFormat = ftype;
 }
 
 QString DateFormatter::dateString(time_t t, const QString &lang, bool shortFormat) const
 {
-    switch (mFormat) {
+    switch (d->mFormat) {
     case Fancy:
-        return fancy(t);
+        return d->fancy(t);
         break;
     case Localized:
-        return localized(t, shortFormat, lang);
+        return d->localized(t, shortFormat, lang);
         break;
     case CTime:
-        return cTime(t);
+        return d->cTime(t);
         break;
     case Iso:
-        return isoDate(t);
+        return d->isoDate(t);
         break;
     case Rfc:
-        return rfc2822(t);
+        return d->rfc2822(t);
         break;
     case Custom:
-        return custom(t);
+        return d->custom(t);
         break;
     }
     return QString();
@@ -95,51 +167,51 @@ QString DateFormatter::dateString(const QDateTime &dt, const QString &lang, bool
     return dateString(dt.toLocalTime().toTime_t(), lang, shortFormat);
 }
 
-QString DateFormatter::rfc2822(time_t t) const
+QString DateFormatterPrivate::rfc2822(time_t t)
 {
     QDateTime tmp;
     QString ret;
 
     tmp.setTime_t(t);
 
-    ret = tmp.toString(QLatin1String("ddd, dd MMM yyyy hh:mm:ss "));
+    ret = tmp.toString(QStringLiteral("ddd, dd MMM yyyy hh:mm:ss "));
     ret += QLatin1String(zone(t));
 
     return ret;
 }
 
-QString DateFormatter::custom(time_t t) const
+QString DateFormatterPrivate::custom(time_t t) const
 {
     if (mCustomFormat.isEmpty()) {
         return QString();
     }
 
     int z = mCustomFormat.indexOf(QLatin1Char('Z'));
-    QDateTime d;
+    QDateTime dt;
     QString ret = mCustomFormat;
 
-    d.setTime_t(t);
+    dt.setTime_t(t);
     if (z != -1) {
         ret.replace(z, 1, QLatin1String(zone(t)));
     }
 
-    ret = d.toString(ret);
+    ret = dt.toString(ret);
 
     return ret;
 }
 
 void DateFormatter::setCustomFormat(const QString &format)
 {
-    mCustomFormat = format;
-    mFormat = Custom;
+    d->mCustomFormat = format;
+    d->mFormat = Custom;
 }
 
 QString DateFormatter::customFormat() const
 {
-    return mCustomFormat;
+    return d->mCustomFormat;
 }
 
-QByteArray DateFormatter::zone(time_t t) const
+QByteArray DateFormatterPrivate::zone(time_t t)
 {
 #if defined(HAVE_TIMEZONE) || defined(HAVE_TM_GMTOFF)
     struct tm *local = localtime(&t);
@@ -148,21 +220,18 @@ QByteArray DateFormatter::zone(time_t t) const
 #if defined(HAVE_TIMEZONE)
 
     //hmm, could make hours & mins static
-    int secs = abs(timezone);
+    int secs = std::abs(timezone);
     int neg  = (timezone > 0) ? 1 : 0;
     int hours = secs / 3600;
     int mins  = (secs - hours * 3600) / 60;
 
     // adjust to daylight
     if (local->tm_isdst > 0) {
-        mDaylight = 1;
         if (neg) {
             --hours;
         } else {
             ++hours;
         }
-    } else {
-        mDaylight = 0;
     }
 
 #elif defined(HAVE_TM_GMTOFF)
@@ -172,12 +241,6 @@ QByteArray DateFormatter::zone(time_t t) const
     int hours = secs / 3600;
     int mins  = (secs - hours * 3600) / 60;
 
-    if (local->tm_isdst > 0) {
-        mDaylight = 1;
-    } else {
-        mDaylight = 0;
-    }
-
 #else
 
     QDateTime d1 = QDateTime::fromString(QString::fromLatin1(asctime(gmtime(&t))));
@@ -187,7 +250,6 @@ QByteArray DateFormatter::zone(time_t t) const
     secs = abs(secs);
     int hours = secs / 3600;
     int mins  = (secs - hours * 3600) / 60;
-    // daylight should be already taken care of here
 
 #endif /* HAVE_TIMEZONE */
 
@@ -200,7 +262,7 @@ QByteArray DateFormatter::zone(time_t t) const
     return ret;
 }
 
-QString DateFormatter::fancy(time_t t) const
+QString DateFormatterPrivate::fancy(time_t t)
 {
     auto locale = QLocale::system();
 
@@ -231,7 +293,7 @@ QString DateFormatter::fancy(time_t t) const
             for (int i = 3; i < 8; i++) {
                 if (diff < i * 24 * 60 * 60) {
                     return i18nc("1. weekday, 2. time", "%1 %2" ,
-                                 locale.toString(old.date(), QLatin1String("dddd")),
+                                 locale.dayName(old.date().dayOfWeek(), QLocale::LongFormat),
                                  locale.toString(old.time(), QLocale::ShortFormat));
                 }
             }
@@ -241,7 +303,7 @@ QString DateFormatter::fancy(time_t t) const
     return locale.toString(old, QLocale::ShortFormat);
 }
 
-QString DateFormatter::localized(time_t t, bool shortFormat, const QString &lang) const
+QString DateFormatterPrivate::localized(time_t t, bool shortFormat, const QString &lang)
 {
     QDateTime tmp;
     QString ret;
@@ -259,23 +321,18 @@ QString DateFormatter::localized(time_t t, bool shortFormat, const QString &lang
     return ret;
 }
 
-QString DateFormatter::cTime(time_t t) const
+QString DateFormatterPrivate::cTime(time_t t)
 {
     return QString::fromLatin1(ctime(&t)).trimmed();
 }
 
-QString DateFormatter::isoDate(time_t t) const
+QString DateFormatterPrivate::isoDate(time_t t)
 {
     char cstr[64];
     strftime(cstr, 63, "%Y-%m-%d %H:%M:%S", localtime(&t));
     return QLatin1String(cstr);
 }
 
-void DateFormatter::reset()
-{
-    mTodayOneSecondBeforeMidnight = 0;
-}
-
 QString DateFormatter::formatDate(FormatType ftype, time_t t,
                                   const QString &data, bool shortFormat)
 {
@@ -294,22 +351,3 @@ QString DateFormatter::formatCurrentDate(FormatType ftype, const QString &data,
     }
     return f.dateString(time(0), data, shortFormat);
 }
-
-bool DateFormatter::isDaylight()
-{
-    if (mDaylight == -1) {
-        time_t ntime = time(0);
-        struct tm *local = localtime(&ntime);
-        if (local->tm_isdst > 0) {
-            mDaylight = 1;
-            return true;
-        } else {
-            mDaylight = 0;
-            return false;
-        }
-    } else if (mDaylight != 0) {
-        return true;
-    } else {
-        return false;
-    }
-}
